@@ -8,18 +8,23 @@ import pytest
 
 from hypothesis import given
 
+from attr._compat import PY_3_14_PLUS
+
 from .strategies import simple_classes
 
 
 cloudpickle = pytest.importorskip("cloudpickle")
 
 
+@pytest.mark.xfail(
+    PY_3_14_PLUS, reason="cloudpickle is currently broken on 3.14."
+)
 class TestCloudpickleCompat:
     """
     Tests for compatibility with ``cloudpickle``.
     """
 
-    @given(simple_classes())
+    @given(simple_classes(cached_property=False))
     def test_repr(self, cls):
         """
         attrs instances can be pickled and un-pickled with cloudpickle.
