@@ -4,8 +4,8 @@
 Tests for dunder methods from `attrib._make`.
 """
 
-
 import copy
+import inspect
 import pickle
 
 import pytest
@@ -19,7 +19,6 @@ from attr._make import (
     NOTHING,
     Factory,
     _add_repr,
-    _is_slot_cls,
     _make_init,
     fields,
     make_class,
@@ -64,16 +63,16 @@ class OrderCallableCSlots:
 # HashC is hashable by explicit definition while HashCSlots is hashable
 # implicitly.  The "Cached" versions are the same, except with hash code
 # caching enabled
-HashC = simple_class(hash=True)
-HashCSlots = simple_class(hash=None, eq=True, frozen=True, slots=True)
-HashCCached = simple_class(hash=True, cache_hash=True)
+HashC = simple_class(unsafe_hash=True)
+HashCSlots = simple_class(unsafe_hash=None, eq=True, frozen=True, slots=True)
+HashCCached = simple_class(unsafe_hash=True, cache_hash=True)
 HashCSlotsCached = simple_class(
-    hash=None, eq=True, frozen=True, slots=True, cache_hash=True
+    unsafe_hash=None, eq=True, frozen=True, slots=True, cache_hash=True
 )
 # the cached hash code is stored slightly differently in this case
 # so it needs to be tested separately
 HashCFrozenNotSlotsCached = simple_class(
-    frozen=True, slots=False, hash=True, cache_hash=True
+    frozen=True, slots=False, unsafe_hash=True, cache_hash=True
 )
 
 
@@ -84,13 +83,20 @@ def _add_init(cls, frozen):
     This function used to be part of _make.  It wasn't used anymore however
     the tests for it are still useful to test the behavior of _make_init.
     """
+    has_pre_init = bool(getattr(cls, "__attrs_pre_init__", False))
+
     cls.__init__ = _make_init(
         cls,
         cls.__attrs_attrs__,
-        getattr(cls, "__attrs_pre_init__", False),
+        has_pre_init,
+        (
+            len(inspect.signature(cls.__attrs_pre_init__).parameters) > 1
+            if has_pre_init
+            else False
+        ),
         getattr(cls, "__attrs_post_init__", False),
         frozen,
-        _is_slot_cls(cls),
+        "__slots__" in cls.__dict__,
         cache_hash=False,
         base_attr_map={},
         is_exc=False,
@@ -436,17 +442,17 @@ class TestAddRepr:
 
 # these are for use in TestAddHash.test_cache_hash_serialization
 # they need to be out here so they can be un-pickled
-@attr.attrs(hash=True, cache_hash=False)
+@attr.attrs(unsafe_hash=True, cache_hash=False)
 class HashCacheSerializationTestUncached:
     foo_value = attr.ib()
 
 
-@attr.attrs(hash=True, cache_hash=True)
+@attr.attrs(unsafe_hash=True, cache_hash=True)
 class HashCacheSerializationTestCached:
     foo_value = attr.ib()
 
 
-@attr.attrs(slots=True, hash=True, cache_hash=True)
+@attr.attrs(slots=True, unsafe_hash=True, cache_hash=True)
 class HashCacheSerializationTestCachedSlots:
     foo_value = attr.ib()
 
@@ -474,12 +480,12 @@ class TestAddHash:
         exc_args = ("Invalid value for hash.  Must be True, False, or None.",)
 
         with pytest.raises(TypeError) as e:
-            make_class("C", {}, hash=1),
+            make_class("C", {}, unsafe_hash=1)
 
         assert exc_args == e.value.args
 
         with pytest.raises(TypeError) as e:
-            make_class("C", {"a": attr.ib(hash=1)}),
+            make_class("C", {"a": attr.ib(hash=1)})
 
         assert exc_args == e.value.args
 
@@ -494,13 +500,18 @@ class TestAddHash:
             "enabled.",
         )
         with pytest.raises(TypeError) as e:
-            make_class("C", {}, hash=False, cache_hash=True)
+            make_class("C", {}, unsafe_hash=False, cache_hash=True)
         assert exc_args == e.value.args
 
         # unhashable case
         with pytest.raises(TypeError) as e:
             make_class(
-                "C", {}, hash=None, eq=True, frozen=False, cache_hash=True
+                "C",
+                {},
+                unsafe_hash=None,
+                eq=True,
+                frozen=False,
+                cache_hash=True,
             )
         assert exc_args == e.value.args
 
@@ -514,7 +525,7 @@ class TestAddHash:
             " init must be True.",
         )
         with pytest.raises(TypeError) as e:
-            make_class("C", {}, init=False, hash=True, cache_hash=True)
+            make_class("C", {}, init=False, unsafe_hash=True, cache_hash=True)
         assert exc_args == e.value.args
 
     @given(booleans(), booleans())
@@ -526,7 +537,7 @@ class TestAddHash:
             "C",
             {"a": attr.ib(hash=False), "b": attr.ib()},
             slots=slots,
-            hash=True,
+            unsafe_hash=True,
             cache_hash=cache_hash,
         )
 
@@ -622,13 +633,13 @@ class TestAddHash:
         Uncached = make_class(
             "Uncached",
             {"hash_counter": attr.ib(factory=HashCounter)},
-            hash=True,
+            unsafe_hash=True,
             cache_hash=False,
         )
         Cached = make_class(
             "Cached",
             {"hash_counter": attr.ib(factory=HashCounter)},
-            hash=True,
+            unsafe_hash=True,
             cache_hash=True,
         )
 
@@ -649,11 +660,11 @@ class TestAddHash:
         Test that the default hash is recalculated after a copy operation.
         """
 
-        kwargs = dict(frozen=frozen, slots=slots, cache_hash=cache_hash)
+        kwargs = {"frozen": frozen, "slots": slots, "cache_hash": cache_hash}
 
         # Give it an explicit hash if we don't have an implicit one
         if not frozen:
-            kwargs["hash"] = True
+            kwargs["unsafe_hash"] = True
 
         @attr.s(**kwargs)
         class C:
@@ -671,7 +682,7 @@ class TestAddHash:
         assert orig_hash != hash(b)
 
     @pytest.mark.parametrize(
-        "klass,cached",
+        ("klass", "cached"),
         [
             (HashCacheSerializationTestUncached, False),
             (HashCacheSerializationTestCached, True),
@@ -704,7 +715,7 @@ class TestAddHash:
         __reduce__ generated when cache_hash=True works in that case.
         """
 
-        @attr.s(frozen=frozen, cache_hash=True, hash=True)
+        @attr.s(frozen=frozen, cache_hash=True, unsafe_hash=True)
         class C:
             x = attr.ib()
 
@@ -829,6 +840,29 @@ class TestAddInit:
         assert [] == i.a
         assert isinstance(i.b, D)
 
+    def test_factory_takes_self(self):
+        """
+        If takes_self on factories is True, self is passed.
+        """
+        C = make_class(
+            "C",
+            {
+                "x": attr.ib(
+                    default=Factory((lambda self: self), takes_self=True)
+                )
+            },
+        )
+
+        i = C()
+
+        assert i is i.x
+
+    def test_factory_hashable(self):
+        """
+        Factory is hashable.
+        """
+        assert hash(Factory(None, False)) == hash(Factory(None, False))
+
     def test_validator(self):
         """
         If a validator is passed, call it with the preliminary instance, the
@@ -935,7 +969,7 @@ class TestNothing:
         assert False is bool(NOTHING)
 
 
-@attr.s(hash=True, order=True)
+@attr.s(unsafe_hash=True, order=True)
 class C:
     pass
 
@@ -944,7 +978,7 @@ class C:
 OriginalC = C
 
 
-@attr.s(hash=True, order=True)
+@attr.s(unsafe_hash=True, order=True)
 class C:
     pass
 
@@ -952,9 +986,11 @@ class C:
 CopyC = C
 
 
-@attr.s(hash=True, order=True)
+@attr.s(unsafe_hash=True, order=True)
 class C:
-    """A different class, to generate different methods."""
+    """
+    A different class, to generate different methods.
+    """
 
     a = attr.ib()
 
